% Copyright 2018, John C. Young, All rights reserved.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function node_list = generate_node_list_by_edges(fmesh,start_data,mesh_order)


   if start_data.element_type == 2 % triangle
       numNodes = (mesh_order+1)*(mesh_order+2)/2;
       isTri = 1;
   elseif start_data.element_type == 3 % quad
       numNodes = (mesh_order+1)^2;
       isTri = 0;
   else
       error('Invalid element type.\n');
   end
       
   node_list = zeros(1,numNodes);
   
   nodes = zeros(mesh_order+1,mesh_order+1);   
   nodes(:,1) = start_data.xnodes;
   nodes(1,:) = start_data.ynodes;   
   
    
   % Stripe the nodes along the 'x'-direction first, then 'y'
   cnt = 1;
   for iy = 1:mesh_order

      node_list(cnt) = nodes(1,iy); cnt = cnt+1;   
      for ix = 1:mesh_order-iy*isTri
          nodes(ix+1,iy+1) = determine_next_node(nodes(ix,iy),nodes(ix+1,iy),nodes(ix,iy+1),fmesh);
          node_list(cnt) = nodes(ix+1,iy); cnt = cnt+1;
      end
      if isTri
         node_list(cnt) = nodes(mesh_order+2-iy,iy); cnt = cnt+1;               
      end
      
   end
   
   if isTri % add the top point
       node_list(cnt) = nodes(1,iy+1);
   else % add top Row for quad
       for ix = 1:mesh_order+1
           node_list(cnt) = nodes(ix,iy+1); cnt = cnt+1;
       end
   end

      
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For triangles, can have either
%       .nY_____.node         .nY_____.node
%       | \      |            |      / |
%       |   \    |     OR     |    /   | 
%       |     \  |            |  /     |
%       .nR_____.nX           .nR_____.nX
%
% For quadrilaterals, can only have
%       .nY_____.node
%       |        |   
%       |        |   
%       |        |   
%       .nR_____.nX  


function node = determine_next_node(nR,nX,nY,fmesh)

   % Find all nodes of edges touching nX and nY excluding nR, nX, nY
   nodeX = adjacent_nodes(nX,[nR,nX,nY],fmesh);
   nodeY = adjacent_nodes(nY,[nR,nX,nY],fmesh);   

   % There should only be one node in both nodeX and nodeY.
   node = intersect(nodeX,nodeY);
   if length(node) ~= 1
      error('Could not find next node.\n');
   end
      
end
% Find the adjacent nodes 
function adjacent = adjacent_nodes(node,exclude,fmesh)
   
   edges = fmesh.get_node_edges(node);

   nodes = zeros(1,2*length(edges));
   i = 1;
   for e = edges
       nodes(i:i+1) = fmesh.get_edge_nodes(e);
       i = i+2;
   end
   adjacent = unique(setdiff(nodes,exclude));
   %adjacent = setdiff(unique(nodes),exclude);   
   
end
