% Copyright 2018, John C. Young, All rights reserved.
%
% Get the starting topology data.
function start_data = get_start_topos(coarse_cell_id,cmesh,fmesh,octree,mesh_order)

   % Get the fine mesh nodes that correspond to the nodes of the coarse mesh element.
   cvnodes = get_fine_mesh_nodes(coarse_cell_id,cmesh,octree);

   % Next, generate the higher-order node lists.  This is done in two parts:
   %    1. Find the fine mesh face that lies inside the coarse mesh cell AND shares the same first node.
   %    2. Use the starting cell info to generate the ordered higher-order node list.
   
   node_s = cvnodes(1); % start node
   node_x = cvnodes(2); % first node along 'x'-direction
   node_y = cvnodes(3); % first node along 'y'-direction
   %node_z = cvnodes(x); % first node along 'z'-direction for volume elements
   faces=fmesh.get_node_faces(node_s);   
      
   start_data = start_topos_surf(fmesh,cmesh.get_element_type(coarse_cell_id),node_s,node_x,node_y,faces,mesh_order);

   start_data.xnodes = sequence_nodes(fmesh,mesh_order,node_s,start_data.edge_x);
   start_data.ynodes = sequence_nodes(fmesh,mesh_order,node_s,start_data.edge_y);   
   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the fine mesh nodes that correspond to the nodes of the coarse mesh element.
function cvnodes = get_fine_mesh_nodes(coarse_cell_id,cmesh,octree)
   coarse_node_ids = cmesh.get_element_nodes(coarse_cell_id);
   coarse_nodes = cmesh.get_node(coarse_node_ids);
   num_cnodes = size(coarse_nodes,2);

   cvnodes = zeros(1,num_cnodes);
   for i = 1:num_cnodes
       cvnodes(i) = octree.inTree(coarse_nodes(:,i));
       if  0 == cvnodes(i) 
           error('Could not find matching fine mesh node.\n');
       end
   end


end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take an edge list and start node and generate an ordered node list
function nodes = sequence_nodes(fmesh,mesh_order,start_node,edges)

   %other_edge_node = @(edge,node) setdiff(fmesh.get_edge_nodes(edge),node);   

   nodes = zeros(1,mesh_order+1);
   nodes(1) = start_node;
   for i = 1:mesh_order
       nodes(i+1) = other_edge_node(edges(i),nodes(i),fmesh);
   end   


end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find the other node of an edge
function onode = other_edge_node(edge,node,mesh)

   enodes = mesh.get_edge_nodes(edge);
   if enodes(1) == node
       onode = enodes(2);
   elseif enodes(2) == node
       onode = enodes(1);
   else
       error('Could not find other node.\n');
   end
       
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
