% Copyright 2018, John C. Young, All rights reserved.
%
function [mesh]=read_mesh(filename)
   
   fid = fopen(filename,'r');
   
   node_index_offset = 1;
   element_index_offset = 1;
   
   % NODES
   node_header = textscan(fid,'%s  %d',1);
   num_nodes = node_header{2};
   
   mesh.nodes = zeros(3,num_nodes);
   node_tmp = fscanf(fid,'%f',[4,num_nodes]);
   
   mesh.nodes(:,node_tmp(1,:)+node_index_offset) = node_tmp(2:4,:);
   clear node_tmp;
 
   % ELEMENTS
   ele_header = textscan(fid,'%s  %d',1);
   num_ele = ele_header{2};

   mesh.element{num_ele} = struct('element_type',0,'element_order',0,'element_nodes',[]);
   mesh.element_nodes = zeros(4,num_ele); % Could make this cell array to handle arbitrary element types better.
   
   for i = 1:num_ele
       eline = fscanf(fid,'%d',3);
       eid = eline(1)+element_index_offset; etype = eline(2); eorder = eline(3);
       if 1 ~= eorder 
           error('Only linear mesh elements are allowed.\n');
       end
       num_ele_nodes = get_num_element_nodes(etype,eorder);
       mesh.element{eid}.nodes = fscanf(fid,'%d',num_ele_nodes)+node_index_offset;  
       mesh.element{eid}.type = etype;
       mesh.element{eid}.order = eorder;
   end

   fclose(fid);

   % Helper functions and data
   % NOTE: Cell array access is often much slower than straight arrays.  
   %       Optimized codes could use simple arrays at the expense code flexibility and coding ease.
   mesh.get_element_nodes = @(eid) mesh.element{eid}.nodes;
   mesh.get_num_element_nodes = @(eid) length(mesh.element{eid}.nodes);
   
   mesh.get_element_order = @(eid) mesh.element{eid}.order;
   mesh.get_element_type  = @(eid) mesh.element{eid}.type;   

   mesh.get_node = @(nid) mesh.nodes(:,nid);   
   mesh.get_num_elements = length(mesh.element);
   mesh.get_num_nodes = size(mesh.nodes,2);   
   
   
   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nnodes = get_num_element_nodes(etype,eorder)
   nnodes = -1;
   if etype == 2 % triangle
       nnodes = ((eorder+1)*(eorder+2))/2;
   elseif etype == 3 % quadrilateral
       nnodes = (eorder+1)^2;
   else
       error('Element type not supported.\n');
   end

end
